import Echo from 'laravel-echo'

<% if (options.broadcaster === 'pusher') { %>
window.Pusher = require('pusher-js')
<% } else if (options.broadcaster === 'socket.io') { %>
window.io = require('socket.io-client')
<% } %>

function getHeaders ({ app }) {
  const headers = {}

  <% if (options.authModule) { %>
  if (app.$auth) {
    const strategy = app.$auth.$storage.state.strategy
    const currentStrategy = app.$auth.strategies[strategy]
    if (currentStrategy) {
      const tokenName = currentStrategy.options.token.name || 'Authorization'
      const token = currentStrategy.token.get()

      if (token) {
        headers[tokenName] = token
      }
    }
  }
  <% } %>

  return headers
}

export default (ctx, inject) => {
  const options = <%= serialize(options) %>
  options.auth = options.auth || {}
  options.auth.headers = {
    ...options.auth.headers,
    ...getHeaders(ctx),
    ...{'content-type': 'application/json'},
    ...{accept: 'application/json'},
  }
  options.authorizer = function(channel, options) {
    return {
      authorize: (socketId, callback) => {
        ctx.$axios.post(options.authEndpoint, {
          socket_id: socketId,
          channel_name: channel.name
        })
        .then(response => {
          callback(false, response.data);
        })
        .catch(error => {
          callback(true, error);
        });
      }
    };
  }

  const echo = new Echo(options)

  if(options.debug) {
    echo.connector.socket.on('connect', function(){
        console.log('[Laravel Echo] connected', echo.socketId());
    });
    echo.connector.socket.on('disconnect', function(){
        console.log('[Laravel Echo] disconnected');
    });
    echo.connector.socket.on('reconnecting', function(attemptNumber){
        console.log('[Laravel Echo] reconnecting', attemptNumber);
    });
    echo.connector.socket.on('connect_error', function(error){
        console.log('[Laravel Echo] connect_error', error);
    });
    echo.connector.socket.on('error', function(error){
        console.log('[Laravel Echo] error', error);
    });
  }

  <% if (options.authModule) { %>
  if (ctx.app.$auth) {
    ctx.app.$auth.$storage.watchState('loggedIn', loggedIn => {
      echo.options.auth.headers = {
        ...echo.options.auth.headers,
        ...getHeaders(ctx)
      }

      if (options.debug) {
          console.log('[Laravel Echo] options:', options)
      }

      <% if (options.connectOnLogin) { %>
      if (loggedIn) {
        echo.connect()
        if (options.debug) {
          console.log('[Laravel Echo] I am connected beacause login called')
        }
      }
      <% } %>

      <% if (options.disconnectOnLogout) { %>
      if (!loggedIn && echo.connector) {
        echo.disconnect()
        if (options.debug) {
          console.log('[Laravel Echo] I am disconnected beacause logged out called')
        }
      }
      <% } %>
    })
  }
  <% } %>

  ctx.$echo = echo
  inject('echo', echo)
}
